define([], function() {
    return function(args) {
        var treeController = {
            leafTypes: [11, 6, 7, 13],
            menuWidget: null,
            navigationController: null,
            prevObject: null, // holds the object prior to navigating away
            currentObjects: null,
			currentNodeObject: null,
            idsInPath: [], // holds all of the node ids from root to the
            // currently selected node
            init: function() {
                // mixin the arguments (menuWidget must be set)
                jQuery.extend(this, args);
                var vc = ViewerController;
                vc.events.navigate.subscribe(this.handleNavigation, this, true);
                var root = vc.root;
                // initialise the checkmarks based on pages that have been
                // previously visited
                this.setCheckmarks(root);
                this.displayListForObject(root);
            },
            isLeaf: function(obj) {
                return (jQuery.inArray(obj.type, this.leafTypes) != -1);
            },
            addTreeNode: function(obj, parentNode, addChildren) {
                /* generate navigation items in the menu widget */
                var newParent = this.menuWidget.generateNavItem(obj, jQuery.proxy(function() {
                    this.nodeTitleClicked(obj);
                }, this), jQuery.proxy(function() {
                    this.nodeIconClicked(obj);
                    this.menuWidget.setBreadcrumbs(this.idsInPath);
                }, this), parentNode, this.isLeaf(obj), obj.checked);
            },
            addTreeNodeChildren: function(obj, parentNode, addChildren) {
                if (obj.children) {
                    this.currentObjects = obj.children;
                    var c;
                    for (var i = 0; i < obj.children.length; i++) {
                        c = obj.children[i];
						// DPV 12/08/15
						// If we enable removal of items if satisfied, then uncomment this line.
                        //if (!ViewerController.isRemoved(c)) {
                            this.addTreeNode(obj.children[i], parentNode, false);
                        //}
                    }
                }
            },
            /* initialise the nodes with whether they should be checked */
            setCheckmarks: function(obj) {
				//if ( !ViewerController.isIdentified(obj) ) return false;
                if (this.isLeaf(obj)) {
                    if (ViewerController.isIdentified(obj)) {
                        obj.checked = true;
                    }
                    return obj.checked;
                } else if (obj.children) {
                    var checked = true;
                    var children = obj.children;
                    children.forEach(jQuery.proxy(function(childObj) {
                        checked = this.setCheckmarks(childObj) && checked;
                    }, this));
                    // if all children are checked, then set checked on this
                    // node too
                    if (checked) {
                        obj.checked = true;
                    }
                    return checked;
                }
                return false;
            },
            /*
             * update the relevant checkmarks after a navigation event by only
             * using the "checked" property we only need to look at one level at
             * a time
             * 
             */
            updateCheckmarksAfterNavigation: function(obj) {
                if (obj && !obj.checked) {
                    obj.checked = true;
					if ( ViewerController.isIdentified(obj) ) {
						this.menuWidget.setCheckmark(obj.id);
					}
                    // updates checkmarks throughout the navigation tree
                    this.setCheckmarks(ViewerController.root);
                }
            },
            handleNavigation: function(type, args) {
                if (this.menuWidget.isInSearch()) return;
                var activeNode = args[0];
                this.idsInPath.length = 0; // clear array
                // generate and display the list on a navigation event
                this.displayListForObject(activeNode);
                // get array of all node ids in path to active node
                while (activeNode.parent) {
                    this.idsInPath.push(activeNode.id);
                    activeNode = activeNode.parent;
                }
                this.idsInPath.push(activeNode.id);
                this.menuWidget.setActiveNavElement(null, args[0].id);
                this.updateCheckmarksAfterNavigation(this.prevObject);
                this.menuWidget.setBreadcrumbs(this.idsInPath);
                this.prevObject = ViewerController.current;
            },
            getFirstLaunchableChild: function(obj) {
                // try to find first launchable child object
                var objList = OSDeliveryEngine.find(obj, this.isLeaf, this);
                if (objList.length > 0 && (objList[0] != obj)) {
                    return objList[0];
                }
                return null;
            },
            navigateBack: function(obj) {
                this.displayListForObject(obj, true);
                this.menuWidget.setBreadcrumbs(this.idsInPath);
            },
            navigateRoot: function() {
				this.currentNodeObject = ViewerController.root;
				this.navigateBack(ViewerController.root);
            },
            nodeIconClicked: function(obj, slideDirection) {
				this.currentNodeObject = obj;
                if (obj.children) {
                    this.displayListForObject(obj.children[0]);
                }
            },
            isObjectInExistingMenu: function(obj) {
                if (this.currentObjects) {
                    for (var i = 0; i < this.currentObjects.length; i++) {
                        if (this.currentObjects[i].id === obj.id) {
                            return true;
                        }
                    }
                }
                return false;
            },
            // generates the appropriate navigation list to show the specified
            // object
            generateListForObject: function(obj, slideDirection) {
                this.menuWidget.createNavigationList(slideDirection);
                if (obj.parent && obj.parent !== ViewerController.root) {
                    this.menuWidget.createBackButton(obj.parent, jQuery.proxy(function() {
                        this.navigateBack(obj.parent);
                    }, this));
                }
                // if at top of hierarchy, show list below else show current level
                var root = obj.parent ? obj.parent : obj;
                var leaf = this.isLeaf(root);
                if (leaf) {
                    this.addTreeNode(root, null, !leaf);
                } else {
                    this.addTreeNodeChildren(root, null, true);
                }
            },
			resetCurrentObjectList: function(){
				this.currentObjects = null;
			},
            displayListForObject: function(obj, slideDirection) {
                /*
                 * check if object is in the current list if not, generate a new
                 * list and display it
                 */
                if (!this.isObjectInExistingMenu(obj)) {
                    this.generateListForObject(obj, slideDirection);
                    this.menuWidget.displayNavigationList();
                }
            },
            nodeTitleClicked: function(obj) {
				this.currentNodeObject = obj;
                /*
                 * if already at the correct level of navigation don't recreate
                 * the navigation menu.
                 */
                if (this.isLeaf(obj)) {
                    if (ViewerController.current === obj) {
                        return;
                    }
					//DPV 12/08/2015. If we the assessment prescription option to disable satisified object, then uncomment this.
					//if ( ViewerController.isEnabled(obj) )
						this.navigationController.navigateTo(obj);
                    return;
                }
                var navItem = null;
                if (!this.isLeaf(obj)) {
                    navItem = this.getFirstLaunchableChild(obj);
                    // if there is nowhere to navigate to, return
                    if (!navItem) {
                        return;
                    }
                } else {
                    navItem = obj;
                }
                this.displayListForObject(navItem);
                if (this.isLeaf(navItem)) {
                    /*
                     * if navigating to the current page but from higher up the
                     * hiearchy we must ensure that the checkmark, active
                     * element, etc. are set
                     */
                    if (ViewerController.current === navItem) {
						//DPV 12/08/2015. If the assessment prescription option to disable satisified object, then uncomment this.
						//if ( ViewerController.isEnabled(obj) )
							this.handleNavigation(null, [navItem]);
                        return;
                    }
                    this.navigationController.navigateTo(navItem);
                }
                return false;
            },
            setActiveNavElement: function(elementClicked) {
                this.menuWidget.setActiveNavElement(elementClicked, ViewerController.current.id);
            }
        };
        jQuery(treeController.init());
        return treeController;
    };
});

